local JITB_ChatAnswer = "|cffffcc00JITB:|r "
local isInitialized = false
local AddonPrefix = "JITB"

--[[
	Sends aa addon message to a player via whisper.
--]]
local function SendAddonMessage(message, player)
	if(message ~= nil and player ~= nil) then
		C_ChatInfo.SendAddonMessage(AddonPrefix, message, "whisper", player);
	end
end

-- Event Frame
local EventFrame = CreateFrame("frame", "EventFrame")
EventFrame:RegisterEvent("PLAYER_LOGIN")
EventFrame:RegisterEvent("CHAT_MSG_ADDON")

--[[
	Scrolls down the list of Battlegrounds via scrollbar clicks and searches the BattlefieldZone containers.
	A maximum of twelve battlegrounds is displayed at once, so we check those twelve possible entries for our id
	on every scroll iteration.
]]
local function SearchBattlegroundById(id)
	local found = false

	-- Scroll to top
	for i = 1, 100, 1 do
		BattlefieldListScrollFrameScrollBarScrollUpButton:Click()
		BattlefieldListScrollFrameScrollBarScrollUpButton:Click()
	end

	--[[
		Scroll through the available battlegrounds programmatically.
		Let's assume there will never be more than 1200 (100 * 12) open battlegrounds.
	--]]
	for i = 1, 100, 1 do
		-- Twelve batteground id frames are always visible at once
		for j = 1, 12, 1 do
			-- Check frame existance, if in any case less than twelve open battlegrounds exist
			if(_G["BattlefieldZone" .. j] ~= nil) then
				local zone = _G["BattlefieldZone" .. j].title

				-- Battleground frame's text is always: {battleground name} {numeric id}
				if(zone ~= nil and string.match(zone, "[0-9]+") ~= nil) then
					local zoneId = string.match(zone, '[0-9]+')

					if(zoneId == id) then
						_G["BattlefieldZone" .. j]:Click()
						found = true

						break;
					end
				end
			end
		end

		if(found == true) then
			break
		end

		BattlefieldListScrollFrameScrollBarScrollDownButton:Click()
		BattlefieldListScrollFrameScrollBarScrollDownButton:Click()
	end

	if(found == false) then
		print(JITB_ChatAnswer .. "Battleground ID not found.")
	end
end

--[[
	Installs an edit box inside the BattlefieldFrame to search the ID via the User Interface.
	Do so only once when opening the window for the first time.
]]
local function ShowSearchField()
	if(isInitialized == false) then
		-- Edit Box
		local SearchByIdField = CreateFrame("EditBox", "SearchByIdField", BattlefieldFrame, "InputBoxTemplate")
		SearchByIdField:SetSize(35, 35)
		SearchByIdField:SetPoint("TOPLEFT", 305, -40)
		SearchByIdField:SetFontObject("GameFontHighlight")
		SearchByIdField:SetTextInsets(0, 0, 0, 1)
		SearchByIdField:SetAutoFocus(false)
		SearchByIdField:SetMaxLetters(3)

		-- Label
		local SearchByIdFieldLabel = CreateFrame("Frame", "SearchByIdFieldLabel", BattlefieldFrame)
		SearchByIdFieldLabel:SetSize(20, 20)
		SearchByIdFieldLabel:SetPoint("TOPLEFT", 278, -47)

		SearchByIdFieldLabel.text = SearchByIdFieldLabel:CreateFontString(nil, "OVERLAY")
		SearchByIdFieldLabel.text:SetPoint("LEFT")
		SearchByIdFieldLabel.text:SetSize(20, 20)
		SearchByIdFieldLabel.text:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
		SearchByIdFieldLabel.text:SetText("ID:")

		-- Search by enter press
		SearchByIdField:SetScript("OnEnterPressed", function(self)
			SearchBattlegroundById(self:GetText())
			SearchByIdField:HighlightText()
		end)

		-- Keep only the numbers inside the edit box
		SearchByIdField:SetScript("OnTextChanged", function(self)
			self:SetText(self:GetText():gsub("%D+", ""))
		end)

		isInitialized = true
	end
end

-- Initialize/show the edit box when showing the BattlefieldFrame
BattlefieldFrame:HookScript("OnShow", ShowSearchField)

--[[
	Tries to extract the battleground id from the minimap tooltip inside a battleground.
--]]
local function GetBattlegroundId()
	if(MiniMapBattlefieldFrame.tooltip ~= nil) then
		local ToolTipText = MiniMapBattlefieldFrame.tooltip
		local bgId = string.match(ToolTipText, "([0-9]+)")

		return bgId
	end

	return nil
end

-- Event Frame handler
EventFrame:SetScript("OnEvent", function(self, event, ...)
	if(event == "PLAYER_LOGIN") then
		C_ChatInfo.RegisterAddonMessagePrefix(AddonPrefix)
	-- Addon messages received
	elseif(event == "CHAT_MSG_ADDON" and ... == AddonPrefix) then
		local prefix, message, channel, sender = ...
		local bgId = string.match(message, '{bgId:"([0-9]+)"}')

		-- Output the received bgId, if not nil
		if(bgId ~= nil) then
			print(JITB_ChatAnswer .. sender .. "'s battleground ID is |cff00ff00" .. bgId .. "|r.")
		end

		-- If the BattlefieldFrame is visible, search the bgId
		if(bgId ~= nil and BattlefieldZone1:IsVisible()) then
			SearchBattlegroundById(bgId)
		-- If the player couldn't find a battleground ID
		elseif(message == "nil") then
			print(JITB_ChatAnswer .. sender .. " currently does not have a battleground ID.")
		-- Whisper your own bgId to the requesting player
		elseif(message == "bgid" and UnitInBattleground("player") ~= nil) then
			local bgId = GetBattlegroundId()

			if(bgId ~= nil) then
				local p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13 = ...
				SendAddonMessage('{bgId:"' .. bgId .. '"}', sender)
			end
		elseif(message == "bgid" and UnitInBattleground("player") == nil) then
			SendAddonMessage("nil", sender)
		end
	end
end)

-- Commands
SLASH_BGID1 = "/bgid"
function SlashCmdList.BGID(msg, editbox)
	-- No arguments, show the help.
	if(msg == "") then
		print(JITB_ChatAnswer .. "Usage: /bgid <id> or /bgid <name>.")
	else
		-- Extract ID and start searching
		if(string.match(msg, "^[0-9]+$")) then
			SearchBattlegroundById(string.match(msg, "^[0-9]+$"))
		-- Take the input string as a character name to request his/her battleground id
		else
			SendAddonMessage("bgid", msg)
		end
	end
end